/*--------------------------------------------------------------------------*\
 |                                                                          |
 |  Copyright (C) 2010                                                      |
 |                                                                          |
 |         , __                 , __                                        |
 |        /|/  \               /|/  \                                       |
 |         | __/ _   ,_         | __/ _   ,_                                | 
 |         |   \|/  /  |  |   | |   \|/  /  |  |   |                        |
 |         |(__/|__/   |_/ \_/|/|(__/|__/   |_/ \_/|/                       |
 |                           /|                   /|                        |
 |                           \|                   \|                        |
 |                                                                          |
 |      Enrico Bertolazzi                                                   |
 |      Dipartimento di Ingegneria Meccanica e Strutturale                  |
 |      Universita` degli Studi di Trento                                   |
 |      Via Mesiano 77, I-38050 Trento, Italy                               |
 |      email: enrico.bertolazzi@unitn.it                                   |
 |                                                                          |
 |      version: 0.1 04-01-2010                                             |
 |                                                                          |
\*--------------------------------------------------------------------------*/

// To avoid double inclusion (old trick)
#ifndef MM_HH
#define MM_HH

#ifndef REAL_TYPE
#define REAL_TYPE double
#endif

#ifndef INTEGER_TYPE
#define INTEGER_TYPE int
#endif

typedef REAL_TYPE    valueType ;
typedef INTEGER_TYPE indexType ;

//#define ROW_MAJOR
#ifdef ROW_MAJOR
  #define ADDR(M,I,J)    M[(I)*ld##M+(J)]
  #define NEXT_COLUMN(M) ++p##M
  #define NEXT_ROW(M)    p##M += ld##M
#else
  #define ADDR(M,I,J)    M[(I)+(J)*ld##M]
  #define NEXT_COLUMN(M) p##M += ld##M
  #define NEXT_ROW(M)    ++p##M
#endif

void
MM_normal( valueType const a[], indexType lda, // n x p
           valueType const b[], indexType ldb, // p x m
           valueType       c[], indexType ldc,
           indexType n,
           indexType p,
           indexType m ) ;

void
MM_temp( valueType const a[], indexType lda, // n x p
         valueType const b[], indexType ldb, // p x m
         valueType       c[], indexType ldc,
         indexType n,
         indexType p,
         indexType m ) ;

void
MM_unroll4( valueType const a[], indexType lda, // n x p
            valueType const b[], indexType ldb, // p x m
            valueType       c[], indexType ldc,
            indexType n,
            indexType p,
            indexType m ) ;

void
MM_unroll8( valueType const a[], indexType lda, // n x p
            valueType const b[], indexType ldb, // p x m
            valueType       c[], indexType ldc,
            indexType n,
            indexType p,
            indexType m ) ;

void
MM_unroll16( valueType const a[], indexType lda, // n x p
             valueType const b[], indexType ldb, // p x m
             valueType       c[], indexType ldc,
             indexType n,
             indexType p,
             indexType m ) ;

void
MM_pointer( valueType const a[], indexType lda, // n x p
            valueType const b[], indexType ldb, // p x m
            valueType       c[], indexType ldc,
            indexType n,
            indexType p,
            indexType m ) ;

void
MM_lam( valueType const a[], indexType lda, // n x p
        valueType const b[], indexType ldb, // p x m
        valueType       c[], indexType ldc,
        indexType n,
        indexType p,
        indexType m ) ;

void
MM_tiling( valueType const a[], indexType lda, // n x p
           valueType const b[], indexType ldb, // p x m
           valueType       c[], indexType ldc,
           indexType n,
           indexType p,
           indexType m,
           indexType step ) ;

void
MM_maeno( valueType const a[], indexType lda, // n x p
          valueType const b[], indexType ldb, // p x m
          valueType       c[], indexType ldc,
          indexType n,
          indexType p,
          indexType m,
          indexType lparm ) ;
          
void
MM_warner( valueType const a[], indexType lda, // n x p
           valueType const b[], indexType ldb, // p x m
           valueType       c[], indexType ldc,
           indexType n,
           indexType p,
           indexType m,
           indexType nb ) ;

void
MM_blas( valueType const a[], indexType lda, // n x p
         valueType const b[], indexType ldb, // p x m
         valueType       c[], indexType ldc,
         indexType n,
         indexType p,
         indexType m ) ;

void
MM_blocking( valueType const a[], indexType lda, // n x p
             valueType const b[], indexType ldb, // p x m
             valueType       c[], indexType ldc,
             indexType n,
             indexType p,
             indexType m,
             indexType step ) ;

void
MM_strassen( valueType const a[], indexType lda, // n x p
             valueType const b[], indexType ldb, // p x m
             valueType       c[], indexType ldc,
             indexType n,
             indexType p,
             indexType m ) ;

#endif

// EOF mm.hh
